#!/bin/sh
##########################################
# BubbleUPnP Server Synology package     #
# created by winxi, inspired by patters  #
# see http://pcloadletter.co.uk/         #
##########################################

BUBBLE_USER="bubbleupnp"
DAEMON_PASS="$(openssl rand 12 -base64 2>nul)"
BUBBLE_HOME="$(cat /etc/passwd | grep "${SYNOPKG_PKGNAME} daemon user" | cut -f6 -d':')"
APP_DATA="appData"

# get the installation volume
appstore_target=$(readlink "${PACKAGE_LOCATION}")
if [ ! -z "${appstore_target}" ]; then
	appstore_dir=$(dirname "${appstore_target}")
	VOLUME=$(dirname "${appstore_dir}")
else
	VOLUME=$(/usr/syno/bin/servicetool --get-alive-volume)
fi

preinst ()
{
	# check for a proper Java installation, a DSM update may have reset /etc/profile to defaults
	source /etc/profile
	if [ -z "$JAVA_HOME" ]; then
		# search for the missing Java path
		counter=1
		while [ -d "/volume${counter}" ];
		do
			JAVA_HOME="/volume${counter}/@appstore/java6/jre"
			if [ -d "$JAVA_HOME" ]; then break
			fi
			JAVA_HOME="/volume${counter}/@appstore/java7/jre"
			if [ -d "$JAVA_HOME" ]; then break
			fi
			JAVA_HOME="/volume${counter}/@appstore/java8/jre"
			if [ -d "$JAVA_HOME" ]; then break
			fi

			let counter=counter+1
			JAVA_HOME=""
		done
	fi
	if [ -z "$JAVA_HOME" -o ! -f "$JAVA_HOME/bin/java" ]; then
		# Java installation couldn't be detected
		echo "Java is not installed or not properly configured. \
		Download and install the Java Synology package."
		exit 1
	fi

	# check for existing user
	USERCHECK=$(synouser --enum all | grep "$BUBBLE_USER")
	if [ "$USERCHECK" == "$BUBBLE_USER" ]; then
		echo "User $BUBBLE_USER already exists. To be able to install BubbleUPnP Server Synology package \
		either remove or rename the existing user." > $SYNOPKG_TEMP_LOGFILE
		exit 1
	fi

	# check for the User home service being enabled
	uh_service_off=userHomeEnable=\"no\"
	uh_service="$(cat /etc/synoinfo.conf | grep $uh_service_off)"
	if [ -n "$uh_service" ]; then
		# the User home service is disabled
		echo "Please enable the user home service (Control Panel -> User -> Advanced)." >> $SYNOPKG_TEMP_LOGFILE
		exit 1
	fi

	exit 0
}

##################################################################################

postinst ()
{
	# create bubbleupnp daemon user
	synouser --add "$BUBBLE_USER" "$DAEMON_PASS" "${SYNOPKG_PKGNAME} daemon user" 0 "" ""

	# determine the bubbleupnp user homedir and save that variable in the user's profile
	BUBBLE_HOME="$(cat /etc/passwd | grep "${SYNOPKG_PKGNAME} daemon user" | cut -f6 -d':')"
	su - "$BUBBLE_USER" -s /bin/sh -c "echo export HOME=${BUBBLE_HOME} >> .profile"
	
	# make required directories
	[ ! -d "$BUBBLE_HOME/$APP_DATA" ] && mkdir "$BUBBLE_HOME/$APP_DATA"
	[ ! -d "$SYNOPKG_PKGDEST/libext" ] && mkdir "$SYNOPKG_PKGDEST/libext"
	[ ! -d "$SYNOPKG_PKGDEST/binext" ] && mkdir "$SYNOPKG_PKGDEST/binext"
	
	# download binaries
	wget https://bubblesoftapps.com/bubbleupnpserver/core/BubbleUPnPServerLauncher.jar -P "${SYNOPKG_PKGDEST}/"
	if [ $? -ne 0 ]; then
		echo "There was a problem downloading BubbleUPnPServerLauncher.jar. \
		Please check your network connection." > $SYNOPKG_TEMP_LOGFILE
		exit 1
	fi
	wget https://bubblesoftapps.com/bubbleupnpserver/core/BubbleUPnPServer.jar -P "${SYNOPKG_PKGDEST}/"
	if [ $? -ne 0 ]; then
		echo "There was a problem downloading BubbleUPnPServer.jar. \
		Please check your network connection." > $SYNOPKG_TEMP_LOGFILE
		exit 1
	fi
	
	# restore configuration.xml if it exists
	PUBLIC_DIR=$(cat /usr/syno/etc/smb.conf | sed -r '/\/'public'$/!d;s/^.*path=(\/volume[0-9]{1,4}\/'public').*$/\1/')
	if [ "${SYNOPKG_PKG_STATUS}" == "INSTALL" -a -f "$PUBLIC_DIR/$BUBBLE_USER/configuration.xml" ]; then
		# use the backup config file
		mv "$PUBLIC_DIR/$BUBBLE_USER/configuration.xml" "$BUBBLE_HOME/$APP_DATA"
		rmdir "$PUBLIC_DIR/$BUBBLE_USER"
		synodsmnotify @administrators "BubbleUPnP Server" "The configuration file saved by a previous uninstallation of BubbleUPnP Server has been restored."
	fi

	# change owner and group of BubbleUPnP folders
	chown -R "$BUBBLE_USER" ${SYNOPKG_PKGDEST}
	chown -R "$BUBBLE_USER" ${BUBBLE_HOME}
	chgrp -R root ${SYNOPKG_PKGDEST}
	chgrp -R users ${BUBBLE_HOME}
	exit 0
}

##################################################################################

preuninst ()
{
	exit 0
}

##################################################################################

postuninst ()
{
	# backup configuration.xml if chosen in the uninstallation dialog
	if [ "$backup_key" == "true" ]; then
		# check if the shared folder 'public' exists
		PUBLIC_DIR=$(cat /usr/syno/etc/smb.conf | sed -r '/\/'public'$/!d;s/^.*path=(\/volume[0-9]{1,4}\/'public').*$/\1/')
		if [ -z "$PUBLIC_DIR" ]; then
			# the shared folder 'public' does not exist
			# create a shared folder 'public', grant read and write privileges to the group 'administrators'
			# and grant read only privileges to the group 'users'
			synoshare --add public "System default shared folder" ${VOLUME}/public "" "@administrators" "@users" 1 0
			PUBLIC_DIR="${VOLUME}/public"
		fi	
		[ ! -d "$PUBLIC_DIR/$BUBBLE_USER" ] && mkdir "$PUBLIC_DIR/$BUBBLE_USER"
		cp -p "$BUBBLE_HOME/$APP_DATA/configuration.xml" "$PUBLIC_DIR/$BUBBLE_USER"
	fi

	synouser --del "$BUBBLE_USER"
	# remove daemon user's home directory (needed since DSM 4.1)
	[ -e /var/services/homes/bubbleupnp ] && rm -r /var/services/homes/bubbleupnp

	exit 0
}

##################################################################################

preupgrade ()
{
	# backup BubbleUPnP Server application data, libext and binext
	[ ! -d "${VOLUME}/@tmp/bubblehome" ] && mkdir ${VOLUME}/@tmp/bubblehome
	[ -d "$BUBBLE_HOME" ] && cp -r "$BUBBLE_HOME"/* ${VOLUME}/@tmp/bubblehome
	
	[ ! -d "${VOLUME}/@tmp/bubblelibext" ] && mkdir ${VOLUME}/@tmp/bubblelibext
	[ -d "$SYNOPKG_PKGDEST/libext" ] && cp -r "$SYNOPKG_PKGDEST"/libext/* ${VOLUME}/@tmp/bubblelibext
	
	[ ! -d "${VOLUME}/@tmp/bubblebinext" ] && mkdir ${VOLUME}/@tmp/bubblebinext
	[ -d "$SYNOPKG_PKGDEST/binext" ] && cp -r "$SYNOPKG_PKGDEST"/binext/* ${VOLUME}/@tmp/bubblebinext

	exit 0
}

##################################################################################

postupgrade ()
{
	# restore the backup of BubbleUPnP Server application data, libext and binext
	if [ -d "${VOLUME}/@tmp/bubblehome" ]; then
		cp -r ${VOLUME}/@tmp/bubblehome/* "$BUBBLE_HOME"
		rm -r ${VOLUME}/@tmp/bubblehome
	fi
	
	if [ -d "${VOLUME}/@tmp/bubblelibext" ]; then
		cp -r ${VOLUME}/@tmp/bubblelibext/* "$SYNOPKG_PKGDEST"/libext
		rm -r ${VOLUME}/@tmp/bubblelibext
	fi
	
	if [ -d "${VOLUME}/@tmp/bubblebinext" ]; then
		cp -r ${VOLUME}/@tmp/bubblebinext/* "$SYNOPKG_PKGDEST"/binext
		rm -r ${VOLUME}/@tmp/bubblebinext
	fi

	synodsmnotify @administrators "BubbleUPnP Server" "A previously installed version of BubbleUPnP Server has been detected. \
    Your existing configuration settings have been preserved."

	# change owner and group of BubbleUPnP folders
	chown -R "$BUBBLE_USER" ${SYNOPKG_PKGDEST}
	chown -R "$BUBBLE_USER" ${BUBBLE_HOME}
	chgrp -R root ${SYNOPKG_PKGDEST}
	chgrp -R users ${BUBBLE_HOME}
	
	exit 0
}
